#ifndef _AclI_H_
#define _AclI_H_

#include <Ice/Ice.h>
#include <IceUtil/IceUtil.h>

#include <string>
#include <iomanip>
#include <iostream>

using namespace std;

#include <Acl/Acl.h>
//#include <DomainService/User.h>

#ifndef ACL_API
#   define ACL_API ICE_DECLSPEC_EXPORT
#endif

class ACL_API AclI : virtual public Acl,virtual public IceUtil::AbstractMutexReadI<IceUtil::RWRecMutex>
{
	public:
	//Constructors for Acl clas
	
	/**
	 * Default constructor
	*/
	AclI();
	
	/**
	 * This constructor create a new Acl an asisngs the id
	*/
	AclI(const Ice::Identity& id);
	
	//interface AclAdmin
	/**
	 * Aade permisos al usuario para ejecutar el metodo con nombre method
	 *
	 * @see OnRezI#delEntity
	*/
	virtual bool addEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Rescinde los permisos al usuario para ejecutar el metodo con nombre method
	 *
	 * @see OnRezI#addEntity
	*/
	
	virtual bool delEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Devueleve una secuencia con los ids de todas las entidades que pueden ejecutar
	 * un metodo.
	 *
	 * @return EntityRefs sequence<Ice::Identity>
	 * @param method nombre del metodo 
	*/
	
	virtual EntityRefs getEntities(const string& method,const Ice::Current&)const;
	
	/**
	 * Devuelve el id correspondiente al propietario del objeto
	 *
	 * return Ice::Identity id del propietario del objeto
	*/
	virtual Ice::Identity getOwmner(const Ice::Current&)const;
	/**
	 * Este metodo cambia el propietario del objeto
	 *
	*/
	virtual bool chown(const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Este metodo se utiliza para autentificar si
	 * existen permisos o no para ejecutar determinada
	 * invocacion de un metodo.
	*/
	virtual bool auth(const Ice::Current&)const;
	
	/**
	 * Establece la ACL Lista de Control de Acesos al valor
	 * por defecto. Es decir establece los permisos iniciales
	 * del objeto.
	*/
	virtual bool setDefaultAcl(const Ice::Current&);

	/**
	 * Aade un metodo a la lista de metodos del objeto
	 * los metodos que se incluyan en esta lista podran ser
	 * chequedos por auth.
	 *
	*/
	virtual bool addMethod(const std::string& method,const Ice::Current&);
	
	/**
	 * Elimina un metodo de la lista de metodos que pueden ser chequeados
	*/
	
	virtual bool delMethod(const std::string& method,const Ice::Current&);
	
	/**
	 * Devuelve la lista de metodos que pueden ser chequeados
	 * cuando se invoca auth
	*/
	
	virtual Strings getMethods(const Ice::Current&)const;
};
#endif
