#ifndef _OnRezI_H_
#define _OnRezI_H_

#include <Ice/Ice.h>
#include <IceUtil/IceUtil.h>
#include <IceStorm/IceStorm.h>

#include <OnRez/OnRez.h>
#include <OnRez/XmlAdmin.h>
#include <OnRez/OnRezWeb.h>
#include <OnRez/OnRezAdmin.h>

#include <EventService/EventListener.h>
#include <EventService/EventPublisher.h>
#include <EventService/EventI.h>

#include <string>
using namespace std;



/**
 * Esta es la clase base para la creacion de servicios
 * distribuidos. En esta clase se implementa una serie de
 * interfaces que definen las funcionalidades bascias de todos
 * los objetos.
 *
 * @author Pepone (pepone@on-rez.com)
 * @short clase base para la creacion de servicios distribuidos
*/
#ifndef ON_REZ_API
#   define ON_REZ_API ICE_DECLSPEC_EXPORT
#endif

class ON_REZ_API  OnRezI : virtual public OnRez , virtual public IceUtil::AbstractMutexReadI<IceUtil::RWRecMutex>
{
	private:
	public:

	static IceStorm::TopicManagerPrx topicManager;
	//Constructors
	
	OnRezI();
	OnRezI(const Ice::Identity& id);
	//interface XmlAdmin

	/**
	 * este metodo devuelve una descripcion XML del objeto
	 *
	 * @return devuelve un std::string que contiene el documento XML
	*/
	virtual std::string getXmlDescription(const Ice::Current&)const=0;

	//interface AclAdmin
	/**
	 * Aade permisos al usuario para ejecutar el metodo con nombre method
	 *
	 * @see OnRezI#delEntity
	*/
	virtual bool addEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Rescinde los permisos al usuario para ejecutar el metodo con nombre method
	 *
	 * @see OnRezI#addEntity
	*/
	
	virtual bool delEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Devueleve una secuencia con los ids de todas las entidades que pueden ejecutar
	 * un metodo.
	 *
	 * @return EntityRefs sequence<Ice::Identity>
	 * @param method nombre del metodo 
	*/
	
	virtual EntityRefs getEntities(const string& method,const Ice::Current&)const;
	
	/**
	 * Devuelve el id correspondiente al propietario del objeto
	 *
	 * return Ice::Identity id del propietario del objeto
	*/
	virtual Ice::Identity getOwmner(const Ice::Current&)const;
	/**
	 * Este metodo cambia el propietario del objeto
	 *
	*/
	virtual bool chown(const Ice::Identity& usuario,const Ice::Current&);
	
	/**
	 * Este metodo se utiliza para autentificar si
	 * existen permisos o no para ejecutar determinada
	 * invocacion de un metodo.
	*/
	virtual bool auth(const Ice::Current&)const;
	
	/**
	 * Establece la ACL Lista de Control de Acesos al valor
	 * por defecto. Es decir establece los permisos iniciales
	 * del objeto.
	*/
	virtual bool setDefaultAcl(const Ice::Current&);

	/**
	 * Aade un metodo a la lista de metodos del objeto
	 * los metodos que se incluyan en esta lista podran ser
	 * chequedos por auth.
	 *
	*/
	virtual bool addMethod(const std::string& method,const Ice::Current&);
	
	/**
	 * Elimina un metodo de la lista de metodos que pueden ser chequeados
	*/
	
	virtual bool delMethod(const std::string& method,const Ice::Current&);
	
	/**
	 * Devuelve la lista de metodos que pueden ser chequeados
	 * cuando se invoca auth
	*/
	
	virtual Strings getMethods(const Ice::Current&)const;

	//interface OnRezAdmin
	
	/**
	 * Asigna el puntero al servicio que contiene el objeto
	*/
	
	virtual bool setService(const OnRezServicePrx& service,const Ice::Current&);
        
	/**
	 * Devuelve un puntero al servicio que crea el objeto
	*/
	
	virtual OnRezServicePrx getService(const Ice::Current&)const;
	
	/**
	 * Devuelve un Prx al obeto
	*/
	virtual OnRezPrx getMyPx(const Ice::Current&)const;
	
	/**
	 * Establece el Prx que se debolvera pra
	 * este objeto.
	*/
	virtual void setMyPx(const OnRezPrx& myPx,const Ice::Current&);
	
	/**
	 * Devuelve el id del objecto actual. Los id son unicos
	 * en todo el sistema de objetos.
	*/
	
	virtual Ice::Identity getId(const Ice::Current&)const;
	
	/**
	 * Asigna el atributo id del objeto
	*/
	virtual bool setId(const Ice::Identity& id,const Ice::Current&);
	/**
	 * Run Type Type Info devueve un entero asociado al Tipo de
	 * objeto es un metodo polimorfico implementaodo en todas
	 * las clases que extiende OnRezI
	*/
	
	virtual int rtti(const Ice::Current&)const;
	
	
	/**
	 * This method asing the Acl object for current object
	*/
	
	virtual bool setAcl(const AclPrx& aclPx,const Ice::Current&);
	
	//interface EventPublisher
	
	/**
	 *
	*/
	virtual void eMit(const Ice::Identity& objectId,const EventPtr& e,const 
Ice::Current&);
	
	//interface EventListener
	
	virtual void suscribe(const Ice::Identity& objectId,const std::string& name,const Ice::Current&);
	
	virtual void unsuscribe(const Ice::Identity& objectId,const std::string& name,const Ice::Current&);
	
	virtual void notify(const Ice::Identity& objectId,const EventPtr& e,const Ice::Current&);
	
	//this virtual function is called when a event is notify
	
	virtual void doEvent(const Ice::Identity& objectId,const EventPtr& e);
	
	
	//interface OnRezWeb
	virtual std::string actionW(const std::string& xmlData,const Ice::Current&);
	virtual std::string actionR(const std::string& xmlData,const Ice::Current&)const;
	
	//interface SearchAdmin
	
	virtual bool addKey(const std::string& key,const Ice::Current&);
	virtual bool delKey(const std::string& key,const Ice::Current&);
	virtual bool muchAny(const Keys& claves,const Ice::Current&)const;
	virtual bool muchAll(const Keys& claves,const Ice::Current&)const;
	
	virtual bool hasKey(const string& key,const Ice::Current&)const;
	virtual Keys getKeys(const Ice::Current&)const;
};
#endif
