
#include <OnRez/OnRezI.h>

//Constructors

IceStorm::TopicManagerPrx OnRezI::topicManager;

OnRezI::OnRezI()
{

}

OnRezI::OnRezI(const Ice::Identity& id)
{
	this->id=id;
}


//interface AclAdmin

bool OnRezI::addEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->addEntity(method,usuario);
}

bool OnRezI::delEntity(const std::string& method,const Ice::Identity& usuario,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->delEntity(method,usuario);
}

EntityRefs OnRezI::getEntities(const string& method,const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return aclPx->getEntities(method);
}

Ice::Identity OnRezI::getOwmner(const Ice::Current&)const
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->getOwmner();
}

bool OnRezI::chown(const Ice::Identity& usuario,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	aclPx->chown(usuario);
	return true;
}

bool OnRezI::auth(const Ice::Current& current)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return aclPx->auth();
}

bool OnRezI::setDefaultAcl(const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->setDefaultAcl();
}

bool OnRezI::addMethod(const std::string& method,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->addMethod(method);
}

bool OnRezI::delMethod(const std::string& method,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return aclPx->delMethod(method);
}

Strings OnRezI::getMethods(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	cout<<"OnRezI getMethods"<<endl;
	return aclPx->getMethods();
}

//interface OnRezAdmin

OnRezPrx OnRezI::getMyPx(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return myPx;
}

void OnRezI::setMyPx(const OnRezPrx& myPx,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	this->myPx=myPx;
}

Ice::Identity OnRezI::getId(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return id;
}

bool OnRezI::setId(const Ice::Identity& id,const Ice::Current&)
{
	IceUtil::RWRecMutex::RLock sync(*this);
	this->id=id;
	return true;
}

int OnRezI::rtti(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return 0;
}

bool OnRezI::setAcl(const AclPrx& aclPx,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	this->aclPx=aclPx;
	return true;
}

bool OnRezI::setService(const OnRezServicePrx& service,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	this->service=service;
	return true;
}
	
OnRezServicePrx OnRezI::getService(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return service;
}


//interface EventPublisher
	
void OnRezI::eMit(const Ice::Identity& objectId,const EventPtr& e,const 
Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	string topicName=Ice::identityToString(objectId)+e->getName();
	try
	{
		topic = OnRezI::topicManager->retrieve(topicName);
	}
	catch(const IceStorm::NoSuchTopic e)
	{
		
		topic = OnRezI::topicManager->create(topicName);
	}
	assert(topic);
	
	Ice::ObjectPrx obj = topic->getPublisher();
	if(!obj->ice_isDatagram())
	{
		obj = obj->ice_oneway();
	}
	EventListenerPrx listener = EventListenerPrx::uncheckedCast(obj);
	listener->notify(objectId,e);
}
	
//interface EventListener
	
void OnRezI::suscribe(const Ice::Identity& objectId,const std::string& name,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	std::string topicName=Ice::identityToString(objectId)+"/"+name;
	try
	{
		IceStorm::TopicPrx newTopic;
		newTopic=OnRezI::topicManager->retrieve(topicName);
		EventListenerPrx listenerPx=EventListenerPrx::uncheckedCast(myPx);
		IceStorm::QoS qos;
		qos["reliability"] = "batch";
		newTopic->subscribe(qos,listenerPx);	
		topics[newTopic->getName()]=newTopic;
	}
	catch(const IceStorm::NoSuchTopic e)
	{
		cout<<"NoSuchTopic"<<endl;
	}
}	

void OnRezI::unsuscribe(const Ice::Identity& objectId,const std::string& name,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	TopicsMap::iterator it;
	std::string topicName=Ice::identityToString(objectId)+"/"+name;
	it=topics.find(topicName);
	if(it!=topics.end())
	{
		try
		{
			IceStorm::TopicPrx newTopic;
			newTopic=topicManager->retrieve(topicName);
			EventListenerPrx listenerPx=EventListenerPrx::uncheckedCast(myPx);
			newTopic->unsubscribe(listenerPx);
		}
		catch(const IceStorm::NoSuchTopic e)
		{
			cout<<"NoSuchTopic"<<endl;
		}
		topics.erase(it);
	}
}

void OnRezI::notify(const Ice::Identity& objectId,const EventPtr& e,const Ice::Current&)
{
	doEvent(objectId,e);
}

//this virtual function is called when a event is notify
	
void OnRezI::doEvent(const Ice::Identity& objectId,const EventPtr& e)
{

}



//interface OnRezWeb

std::string OnRezI::actionW(const std::string& xmlData,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	return "<OnRezI::actionW/>";
}


std::string OnRezI::actionR(const std::string& xmlData,const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return "<OnRezI::actionR/>";
}

//interface SearchAdmin

bool OnRezI::addKey(const std::string& key,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	bool retval=false;
	Keys::const_iterator it;
	it=find(claves.begin(),claves.end(),key);
	if(it==claves.end())
	{
		claves.push_back(key);
		retval=true;
	}
	return retval;
}

bool OnRezI::delKey(const std::string& key,const Ice::Current&)
{
	IceUtil::RWRecMutex::WLock sync(*this);
	bool retval=false;
	Keys::iterator it;
	it=find(claves.begin(),claves.end(),key);
	if(it!=claves.end())
	{
		claves.erase(it);
		retval=true;
	}
	return retval;
}

bool OnRezI::muchAny(const Keys& search,const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	bool found=false;
	Keys::const_iterator itSearch;
	Keys::const_iterator itClaves;
	for(itSearch=search.begin();(!found)&&(itSearch!=search.end());itSearch++)
	{
		itClaves=find(claves.begin(),claves.end(),(*itSearch));
		if(itClaves!=claves.end())
			found=true;
	}
	return found;
}

bool OnRezI::muchAll(const Keys& search,const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	bool foundAll=false;
	bool found=true;
	Keys::const_iterator itSearch;
	Keys::const_iterator itClaves;
	for(itSearch=search.begin();(found)&&(itSearch!=search.end());)
	{
		found=false;
		itClaves=find(claves.begin(),claves.end(),(*itSearch));
		itSearch++;
		if(itClaves!=claves.end())
		{
			found=true;
			if(itSearch==search.end())
				foundAll=true;
		}
	}
	return found;
}


bool OnRezI::hasKey(const string& key,const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	bool found=false;
	Keys::const_iterator itClaves;

	itClaves=find(claves.begin(),claves.end(),key);
	if(itClaves!=claves.end())
		found=true;
	
	return found;
}


Keys OnRezI::getKeys(const Ice::Current&)const
{
	IceUtil::RWRecMutex::RLock sync(*this);
	return claves;
}	